@@ -1,5 +1,6 @@
 from chalk.features import Cron, Environments, Tags, description, is_primary, owner, tags
 from chalk.features.resolver import OfflineResolver, OnlineResolver, Resolver, offline, online
+from chalk.logging import chalk_logger
 from chalk.state import State
 from chalk.streams import stream
 from chalk.utils.duration import ScheduleOptions
@@ -23,6 +24,7 @@ __all__ = [
     "stream",
     "tags",
     "Resolver",
+    "chalk_logger",
     "OnlineResolver",
     "OfflineResolver",
 ]
