@@ -1,9 +1,9 @@
 from typing import Optional
 
 from chalk.client.client_impl import ChalkAPIClientImpl
-from chalk.client.client_protocol import (
-    ChalkAPIClientProtocol,
-    ChalkBaseException,
+from chalk.client.client_protocol import ChalkAPIClientProtocol, OnlineQueryResponseProtocol
+from chalk.client.exc import ChalkBaseException
+from chalk.client.models import (
     ChalkError,
     ChalkException,
     ErrorCode,
@@ -12,6 +12,7 @@ from chalk.client.client_protocol import (
     OfflineQueryContext,
     OnlineQueryContext,
     OnlineQueryResponse,
+    ResolverRunResponse,
     WhoAmIResponse,
 )
 
@@ -20,10 +21,12 @@ __all__ = [
     "FeatureResult",
     "WhoAmIResponse",
     "OnlineQueryResponse",
+    "OnlineQueryResponseProtocol",
     "ChalkBaseException",
     "ChalkAPIClientProtocol",
     "OnlineQueryContext",
     "OfflineQueryContext",
+    "ResolverRunResponse",
     "ErrorCode",
     "ErrorCodeCategory",
     "ChalkException",
