@@ -3,7 +3,7 @@ from __future__ import annotations
 import collections.abc
 import time
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Any, Iterable, List, Mapping, Optional, Sequence, Type, TypeVar, Union
+from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Sequence, Type, TypeVar, Union
 from urllib.parse import urljoin
 
 import pandas as pd
@@ -12,18 +12,28 @@ from pydantic import BaseModel, ValidationError
 from requests import HTTPError
 
 from chalk._version import __version__ as chalkpy_version
-from chalk.client.client_protocol import (
-    ChalkAPIClientProtocol,
-    ChalkBaseException,
+from chalk.client.client_protocol import ChalkAPIClientProtocol, OnlineQueryResponseProtocol
+from chalk.client.dataset import DatasetVersion, load_dataset
+from chalk.client.exc import ChalkOfflineQueryException, ChalkResolverRunException
+from chalk.client.models import (
     ChalkError,
+    ComputeResolverOutputRequest,
+    ComputeResolverOutputResponse,
+    CreateOfflineQueryJobRequest,
+    CreateOfflineQueryJobResponse,
+    ExchangeCredentialsRequest,
+    ExchangeCredentialsResponse,
     FeatureResult,
+    GetOfflineQueryJobResponse,
     OfflineQueryContext,
+    OfflineQueryInput,
     OnlineQueryContext,
+    OnlineQueryRequest,
     OnlineQueryResponse,
     ResolverRunResponse,
+    TriggerResolverRunRequest,
     WhoAmIResponse,
 )
-from chalk.client.dataset import DatasetVersion, load_dataset
 from chalk.config.auth_config import load_token
 from chalk.features import DataFrame, Feature, FeatureWrapper, ensure_feature
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE
@@ -36,49 +46,6 @@ if TYPE_CHECKING:
 
 _logger = get_logger(__name__)
 
-
-class _ExchangeCredentialsRequest(BaseModel):
-    client_id: str
-    client_secret: str
-    grant_type: str
-
-
-class _ExchangeCredentialsResponse(BaseModel):
-    access_token: str
-    token_type: str
-    expires_in: int
-    # expires_at: datetime
-    api_server: str
-    primary_environment: Optional[str]
-    engines: Optional[Mapping[str, str]] = None
-
-
-class _OfflineQueryResponse(BaseModel):
-    columns: List[str]
-    output: List[List[Any]]
-    errors: Optional[List[ChalkError]]
-
-
-class _OfflineQueryInput(BaseModel):
-    columns: List[str]
-    values: List[List[Any]]
-
-
-class _QueryRequest(BaseModel):
-    inputs: Mapping[str, Any]
-    outputs: List[str]
-    staleness: Optional[Mapping[str, str]] = None
-    context: Optional[OnlineQueryContext]
-    deployment_id: Optional[str] = None
-    correlation_id: Optional[str] = None
-    query_name: Optional[str] = None
-    meta: Optional[Mapping[str, str]] = None
-
-
-class _TriggerResolverRunRequest(BaseModel):
-    resolver_fqn: str
-
-
 T = TypeVar("T")
 
 
@@ -89,104 +56,7 @@ class _ChalkClientConfig(BaseModel):
     active_environment: Optional[str]
 
 
-class _OnlineQueryResponse(BaseModel):
-    data: List[FeatureResult]
-    errors: Optional[List[ChalkError]]
-
-
-class _CreateOfflineQueryJobRequest(BaseModel):
-    """
-    Attributes
-        output: A list of output feature root fqns to query
-        destination_format: The desired output format. Should be 'CSV' or 'PARQUET'
-        input: Any givens
-        max_samples: The maximum number of samples
-    """
-
-    output: List[str]
-    required_output: List[str]
-    destination_format: str
-    input: Optional[_OfflineQueryInput] = None
-    max_samples: Optional[int] = None
-    observed_at_lower_bound: Optional[datetime] = None
-    observed_at_upper_bound: Optional[datetime] = None
-    dataset_name: Optional[str] = None
-    branch: Optional[str] = None
-    environment: Optional[str] = None
-
-
-# FIXME document
-class _ComputeResolverOutputRequest(BaseModel):
-    input: _OfflineQueryInput
-    resolver_fqn: str
-    branch: Optional[str] = None
-    environment: Optional[str] = None
-
-
-class _RecomputeResolverOutputRequest(BaseModel):
-    persistent_id: str
-    resolver_fqn: str
-    branch: Optional[str] = None
-    environment: Optional[str] = None
-
-
-class _ComputeResolverOutputResponse(BaseModel):
-    job_id: str
-    persistent_id: str
-    errors: Optional[List[ChalkError]]
-
-
-class _CreateOfflineQueryJobResponse(BaseModel):
-    """
-    Attributes:
-        job_id: A job ID, which can be used to retrieve the results.
-    """
-
-    job_id: str
-    version: int = 1  # Deprecated
-    errors: Optional[List[ChalkError]]
-
-
-class _GetOfflineQueryJobResponse(BaseModel):
-    """
-    Attributes:
-        is_finished: Whether the export job is finished (it runs asynchronously)
-        signed_urls: A list of signed URLs that the client can download to retrieve the exported data.
-    """
-
-    is_finished: bool
-    version: int
-    urls: List[str]
-    errors: Optional[List[ChalkError]]
-
-
-class ChalkOfflineQueryException(ChalkBaseException):
-    message: str
-    errors: List[ChalkError]
-
-    def __init__(self, message: str, errors: List[ChalkError]):
-        self.message = message
-        self.errors = errors
-        super().__init__(message + "\n" + "\n".join(["\t" + e.message for e in errors[0:3]]))
-
-
-class ChalkResolverRunException(ChalkBaseException):
-    message: str
-
-    def __init__(self, message: str):
-        self.message = message
-        super().__init__(message)
-
-
-class ChalkDatasetDownloadException(ChalkBaseException):
-    message: str
-
-    def __init__(self, message: str):
-        self.message = message
-        super().__init__(message)
-
-
-class OnlineQueryResponseWrapper(OnlineQueryResponse):
+class OnlineQueryResponseImpl(OnlineQueryResponseProtocol):
     data: List[FeatureResult]
     errors: List[ChalkError]
     warnings: List[str]
@@ -206,13 +76,18 @@ class OnlineQueryResponseWrapper(OnlineQueryResponse):
 
         self._values = {d.field: d for d in self.data}
 
+    def _df_repr(self):
+        if self.errors:
+            info = [vars(x) for x in self.errors]
+        else:
+            info = [{"field": x.field, "value": x.value, "error": x.error, "ts": x.ts} for x in self.data]
+        return pd.DataFrame(info)
+
     def __repr__(self) -> str:
-        info = self.errors if self.errors else self.data
-        return repr(pd.DataFrame([vars(x) for x in info]))
+        return repr(self._df_repr())
 
     def _repr_html_(self) -> str | None:
-        info = self.errors if self.errors else self.data
-        return pd.DataFrame([vars(x) for x in info])._repr_html_()
+        return self._df_repr()._repr_html_()
 
     def get_feature(self, feature: Any) -> Optional[FeatureResult]:
         # Typing `feature` as Any, as the Features will be typed as the underlying datatypes, not as Feature
@@ -276,7 +151,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
                 "Accept": "application/json",
                 "Content-Type": "application/json",
             },
-            json=_ExchangeCredentialsRequest(
+            json=ExchangeCredentialsRequest(
                 client_id=self._config.client_id,
                 client_secret=self._config.client_secret,
                 grant_type="client_credentials",
@@ -286,7 +161,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         resp.raise_for_status()
         response_json = resp.json()
         try:
-            creds = _ExchangeCredentialsResponse(**response_json)
+            creds = ExchangeCredentialsResponse(**response_json)
         except ValidationError:
             raise HTTPError(response=resp)
         self._default_headers["Authorization"] = f"Bearer {creds.access_token}"
@@ -305,9 +180,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         method: str,
         uri: str,
         response: Type[T],
-        json: Optional[BaseModel] = None,
-        use_engine: bool = False,
-        environment_override: Optional[str] = None,
+        json: Optional[BaseModel],
+        use_engine: bool,
+        environment_override: Optional[str],
     ) -> T:
         # Track whether we already exchanged credentials for this request
         exchanged_credentials = False
@@ -328,7 +203,14 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         return response(**r.json())
 
     def whoami(self) -> WhoAmIResponse:
-        return self._request(method="GET", uri=f"/v1/who-am-i", response=WhoAmIResponse)
+        return self._request(
+            method="GET",
+            uri=f"/v1/who-am-i",
+            response=WhoAmIResponse,
+            json=None,
+            use_engine=False,
+            environment_override=None,
+        )
 
     def upload_features(
         self,
@@ -360,7 +242,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
         meta: Optional[Mapping[str, str]] = None,
-    ) -> OnlineQueryResponse:
+    ) -> OnlineQueryResponseProtocol:
         warnings: List[str] = []
         encoded_inputs = {}
         for feature, value in input.items():
@@ -372,7 +254,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
                 encoded_inputs[feature] = value
         outputs: List[str] = [str(feature) for feature in output]
 
-        request = _QueryRequest(
+        request = OnlineQueryRequest(
             inputs=encoded_inputs,
             outputs=outputs,
             staleness={} if staleness is None else {ensure_feature(k).root_fqn: v for k, v in staleness.items()},
@@ -387,11 +269,11 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             method="POST",
             uri="/v1/query/online",
             json=request,
-            response=_OnlineQueryResponse,
+            response=OnlineQueryResponse,
             use_engine=preview_deployment_id is None,
             environment_override=context.environment if context else None,
         )
-        return OnlineQueryResponseWrapper(data=resp.data, errors=resp.errors or [], warnings=warnings)
+        return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=warnings)
 
     def get_training_dataframe(
         self,
@@ -404,7 +286,10 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         dataset: Optional[str] = None,
         branch: Optional[str] = None,
         max_samples: Optional[int] = None,
+        disallow_stale_results: bool = False,
     ) -> pd.DataFrame:
+        if context is None:
+            context = OfflineQueryContext()
         try:
             import polars as pl
         except ImportError:
@@ -421,8 +306,6 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         if len(output) == 0 and len(required_output) == 0:
             raise ValueError("Either 'output' or 'required_output' must be specified.")
 
-        self._get_headers(environment_override=None)
-
         return self._get_training_dataframe(
             input=input,
             input_times=input_times,
@@ -434,6 +317,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             dataset=dataset,
             branch=branch,
             max_samples=max_samples,
+            disallow_stale_results=disallow_stale_results,
         )
 
     def _get_training_dataframe(
@@ -445,9 +329,10 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         output_id: bool,
         output_ts: bool,
         max_samples: Optional[int],
-        context: Optional[OfflineQueryContext],
+        context: OfflineQueryContext,
         dataset: Optional[str],
         branch: Optional[str],
+        disallow_stale_results: bool,
     ) -> pd.DataFrame:
         columns = input.columns
         matrix: List[List[Any]] = input.T.values.tolist()
@@ -473,7 +358,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
                 matrix[col_index][row_index] = encoded_feature
 
-        query_input = _OfflineQueryInput(
+        query_input = OfflineQueryInput(
             columns=columns_fqn,
             values=matrix,
         )
@@ -486,8 +371,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             output_ts=output_ts,
             dataset_name=dataset,
             branch=branch,
+            context=context,
             max_samples=max_samples,
-            environment=context and context.environment,
+            disallow_stale_results=disallow_stale_results,
         )
         return response.to_pandas()
 
@@ -501,7 +387,10 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         dataset: Optional[str] = None,
         branch: Optional[str] = None,
         context: Optional[OfflineQueryContext] = None,
+        disallow_stale_results: bool = False,
     ) -> pd.DataFrame:
+        if context is None:
+            context = OfflineQueryContext()
         optional_output_root_fqns = [str(f) for f in output]
         required_output_root_fqns = [str(f) for f in required_output]
 
@@ -514,11 +403,12 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
                 optional_output=optional_output_root_fqns,
                 required_output=required_output_root_fqns,
                 max_samples=max_samples,
+                context=context,
                 output_id=output_id,
                 output_ts=output_ts,
                 dataset_name=dataset,
                 branch=branch,
-                environment=context and context.environment,
+                disallow_stale_results=disallow_stale_results,
             )
         except HTTPError as e:
             _logger.debug("Got HTTP Exception while processing sample query", exc_info=e)
@@ -529,17 +419,21 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
     def trigger_resolver_run(
         self,
         resolver_fqn: str,
-        deployment_id: Optional[str] = None,
+        context: Optional[OfflineQueryContext] = None,
     ) -> ResolverRunResponse:
-        _logger.debug(f'Triggering resolver {resolver_fqn} to run with deployment ID "{deployment_id}"')
+        _logger.debug(f"Triggering resolver {resolver_fqn} to run")
+        if context is None:
+            context = OfflineQueryContext()
 
-        request = _TriggerResolverRunRequest(resolver_fqn=resolver_fqn)
+        request = TriggerResolverRunRequest(resolver_fqn=resolver_fqn)
         try:
             response = self._request(
                 method="POST",
                 uri="/v1/runs/trigger",
                 json=request,
                 response=ResolverRunResponse,
+                use_engine=False,
+                environment_override=context.environment,
             )
         except HTTPError as e:
 
@@ -551,12 +445,17 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
         return response
 
-    def get_run_status(self, run_id: str) -> ResolverRunResponse:
+    def get_run_status(self, run_id: str, context: Optional[OfflineQueryContext] = None) -> ResolverRunResponse:
+        if context is None:
+            context = OfflineQueryContext()
         try:
             response = self._request(
                 method="GET",
                 uri=f"/v1/runs/{run_id}",
                 response=ResolverRunResponse,
+                json=None,
+                use_engine=False,
+                environment_override=context.environment,
             )
         except HTTPError as e:
 
@@ -572,15 +471,16 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         self,
         optional_output: List[str],
         required_output: List[str],
-        query_input: Optional[_OfflineQueryInput],
+        query_input: Optional[OfflineQueryInput],
         max_samples: Optional[int],
         dataset_name: Optional[str],
         branch: Optional[str],
-        environment: Optional[str],
+        context: OfflineQueryContext,
         output_id: bool,
         output_ts: bool,
+        disallow_stale_results: bool,
     ) -> pl.DataFrame:
-        req = _CreateOfflineQueryJobRequest(
+        req = CreateOfflineQueryJobRequest(
             output=optional_output,
             required_output=required_output,
             destination_format="PARQUET",
@@ -588,15 +488,15 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             max_samples=max_samples,
             dataset_name=dataset_name,
             branch=branch,
-            environment=environment,
+            max_cache_age_secs=0 if disallow_stale_results else None,
         )
-        response = self._create_offline_query_job(request=req)
+        response = self._create_offline_query_job(request=req, context=context)
 
         if response.errors is not None and len(response.errors) > 0:
             raise ChalkOfflineQueryException(message="Failed to execute offline query", errors=response.errors)
 
         while True:
-            status = self._get_job_status(job_id=response.job_id)
+            status = self._get_job_status(job_id=response.job_id, context=context)
             if status.is_finished:
                 break
             time.sleep(0.5)
@@ -613,7 +513,10 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         input: Union[Mapping[Union[str, Feature], Any], pl.DataFrame, pd.DataFrame, DataFrame],
         input_times: List[datetime],
         resolver: str,
+        context: Optional[OfflineQueryContext] = None,
     ) -> pl.DataFrame:
+        if context is None:
+            context = OfflineQueryContext()
         if not isinstance(input, DataFrame):
             input = DataFrame(input)
         input = input.to_pandas()
@@ -632,36 +535,58 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
                     value=value,
                 )
 
-        query_input = _OfflineQueryInput(
+        query_input = OfflineQueryInput(
             columns=columns_fqn,
             values=matrix,
         )
-        request = _ComputeResolverOutputRequest(input=query_input, resolver_fqn=resolver)
+        request = ComputeResolverOutputRequest(input=query_input, resolver_fqn=resolver)
         response = self._request(
-            method="POST", uri="/v1/compute_resolver_output", json=request, response=_ComputeResolverOutputResponse
+            method="POST",
+            uri="/v1/compute_resolver_output",
+            json=request,
+            response=ComputeResolverOutputResponse,
+            environment_override=context.environment,
+            use_engine=False,
         )
 
         if response.errors is not None and len(response.errors) > 0:
             raise ChalkOfflineQueryException(message="Failed to execute offline query", errors=response.errors)
 
         while True:
-            status = self._get_compute_job_status(job_id=response.job_id)
+            status = self._get_compute_job_status(job_id=response.job_id, context=context)
             if status.is_finished:
                 break
             time.sleep(0.5)
 
-        return load_dataset(uris=status.urls, version=DatasetVersion.COMPUTE_RESOLVER_OUTPUT_V1, executor=None)
+        return load_dataset(uris=status.urls, version=status.version, executor=None)
 
-    def _get_compute_job_status(self, job_id: str) -> _GetOfflineQueryJobResponse:
+    def _get_compute_job_status(self, job_id: str, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
         return self._request(
-            method="GET", uri=f"/v1/compute_resolver_output/{job_id}", response=_GetOfflineQueryJobResponse
+            method="GET",
+            uri=f"/v1/compute_resolver_output/{job_id}",
+            response=GetOfflineQueryJobResponse,
+            json=None,
+            use_engine=False,
+            environment_override=context.environment,
         )
 
-    def _create_offline_query_job(self, request: _CreateOfflineQueryJobRequest):
+    def _create_offline_query_job(self, request: CreateOfflineQueryJobRequest, context: OfflineQueryContext):
         response = self._request(
-            method="POST", uri="/v2/offline_query", json=request, response=_CreateOfflineQueryJobResponse
+            method="POST",
+            uri="/v2/offline_query",
+            json=request,
+            response=CreateOfflineQueryJobResponse,
+            environment_override=context.environment,
+            use_engine=False,
         )
         return response
 
-    def _get_job_status(self, job_id: str) -> _GetOfflineQueryJobResponse:
-        return self._request(method="GET", uri=f"/v2/offline_query/{job_id}", response=_GetOfflineQueryJobResponse)
+    def _get_job_status(self, job_id: str, context: OfflineQueryContext) -> GetOfflineQueryJobResponse:
+        return self._request(
+            method="GET",
+            uri=f"/v2/offline_query/{job_id}",
+            response=GetOfflineQueryJobResponse,
+            environment_override=context.environment,
+            use_engine=False,
+            json=None,
+        )
