@@ -9,6 +9,8 @@ from concurrent.futures import Future, ThreadPoolExecutor
 from enum import IntEnum
 from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Sequence, Union
 
+from typing_extensions import assert_never
+
 from chalk.features import Feature, FeatureWrapper, ensure_feature
 from chalk.utils.collections import get_unique_item
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -240,7 +242,7 @@ def _load_dataset_bigquery(
 
 def load_dataset(
     uris: List[str],
-    version: DatasetVersion,
+    version: DatasetVersion | int,
     output_features: Optional[Sequence[Union[str, Feature, FeatureWrapper, Any]]] = None,
     output_id: bool = True,
     output_ts: bool = True,
@@ -251,6 +253,15 @@ def load_dataset(
     except ImportError:
         raise missing_dependency_exception("chalkpy[polars]")
     del pl  # Unused
+    if not isinstance(version, DatasetVersion):
+        try:
+            version = DatasetVersion(version)
+        except ValueError:
+            raise ValueError(
+                f"The dataset version ({version}) is not supported by this installed version of the Chalk client. "
+                "Please upgrade your chalk client and try again."
+            )
+
     if version == DatasetVersion.DATASET_WRITER:
         return _load_dataset_from_chalk_writer(uris)
     output_feature_fqns = (
@@ -271,6 +282,4 @@ def load_dataset(
             output_id=output_id,
             output_ts=output_ts,
         )
-    raise ValueError(
-        f"The dataset version ({version}) is not supported by this installed version of the Chalk client. Please upgrade your chalk client and try again."
-    )
+    assert_never(version)
