@@ -256,7 +256,13 @@ def _get_field(
 
     # Only at this point do we know the name and the type.  Set them.
     f.namespace = namespace
-    f.typ = ParsedAnnotation(cls, annotation_name)
+    typ = ParsedAnnotation(cls, annotation_name)
+    f.typ = typ
+    if typ.is_feature_time is True:
+        f.is_feature_time = True
+    if typ.is_primary is True:
+        f.primary = True
+
     f.features_cls = cls
     f.attribute_name = annotation_name
     _process_field(f, comments, class_owner, class_tags, class_etl_offline_to_online, class_max_staleness)
