@@ -117,6 +117,11 @@ class Resolver(Generic[P, T]):
     registry: "ClassVar[List[Resolver]]" = []
     hook: "ClassVar[Optional[Callable[[Resolver], None]]]" = None
 
+    def __eq__(self, other: object):
+        if not isinstance(other, type(self)):
+            return NotImplemented
+        return isinstance(other, type(self)) and self.fqn == other.fqn
+
     def __hash__(self):
         return hash(self.fqn)
 
@@ -146,9 +151,6 @@ def _process_call(result: T, *, declared_output: Optional[Type[Features]]) -> T:
 class SinkResolver(Resolver[P, T]):
     registry: "List[SinkResolver]" = []
 
-    def __eq__(self, other):
-        return isinstance(other, SinkResolver) and self.fqn == other.fqn
-
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
         return _process_call(self.fn(*args, **kwargs), declared_output=None)
 
@@ -201,9 +203,6 @@ class SinkResolver(Resolver[P, T]):
 class OnlineResolver(Resolver[P, T]):
     cron: Union[ScheduleOptions, Cron]
 
-    def __eq__(self, other):
-        return isinstance(other, OnlineResolver) and self.fqn == other.fqn
-
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
         return _process_call(self.fn(*args, **kwargs), declared_output=self.output)
 
@@ -251,9 +250,6 @@ class OnlineResolver(Resolver[P, T]):
 
 
 class OfflineResolver(Resolver[P, T]):
-    def __eq__(self, other):
-        return isinstance(other, OfflineResolver) and self.fqn == other.fqn
-
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:
         return _process_call(self.fn(*args, **kwargs), declared_output=self.output)
 
