@@ -4,7 +4,7 @@ import collections.abc
 import dataclasses
 import enum
 from datetime import date, datetime, timezone
-from typing import TYPE_CHECKING, Any, List, Type, Union, cast, get_type_hints
+from typing import TYPE_CHECKING, Any, Dict, List, Type, Union, cast, get_type_hints
 
 import cattrs
 import pendulum
@@ -28,6 +28,11 @@ if TYPE_CHECKING:
     import polars as pl
 
     from chalk.features import Feature
+else:
+    try:
+        import polars as pl
+    except ImportError:
+        pl = None
 
 try:
     import attrs
@@ -95,6 +100,7 @@ class FeatureCodec:
                 cast(datetime, v) if cast(datetime, v).tzinfo else pytz.utc.localize(cast(datetime, v))
             ).isoformat(),
         )
+        self._polars_dtype_cache: Dict[str, Union[pl.DataType, Type[pl.DataType]]] = {}
 
     def _default_encode(self, value: Any):
         if isinstance(value, set):
@@ -265,9 +271,10 @@ class FeatureCodec:
         return "object"
 
     def get_polars_dtype(self, fqn: str) -> Union[pl.DataType, Type[pl.DataType]]:
-        try:
-            import polars as pl
-        except ImportError:
+        if fqn in self._polars_dtype_cache:
+            return self._polars_dtype_cache[fqn]
+
+        if pl is None:
             raise missing_dependency_exception("chalkpy[polars]")
 
         # BEGIN This should be replaced with a structification
@@ -277,15 +284,18 @@ class FeatureCodec:
         # END structification workaround
 
         if feature.is_has_one or feature.is_has_many:
-            return pl.Object
-        pa_type = self.get_pyarrow_dtype(fqn)
+            typ = pl.Object
+        else:
+            pa_type = self.get_pyarrow_dtype(fqn)
+            try:
+                typ = self._pa_to_pl(pa_type)
+            except Exception as exc:
+                raise ValueError(
+                    f"Failed to determine type for feature '{fqn}'. Please set the pyarrow_dtype attribute directly."
+                ) from exc
 
-        try:
-            return self._pa_to_pl(pa_type)
-        except Exception as exc:
-            raise ValueError(
-                f"Failed to determined type for feature {fqn}. Please set the pyarrow_dtype attribute directly."
-            ) from exc
+        self._polars_dtype_cache[fqn] = typ
+        return typ
 
     def _pa_to_pl(self, pa_type: pa.DataType) -> Union[pl.DataType, Type[pl.DataType]]:
         try:
