@@ -1,18 +1,22 @@
-import tempfile
 from os import PathLike
 from typing import Optional, Union, overload
 
-from chalk.sql.base.protocols import BaseSQLSourceProtocol, IncrementalSettings, TableIngestProtocol
-from chalk.sql.base.sql_source import BaseSQLSource
-from chalk.sql.integrations.bigquery import BigQuerySourceImpl
-from chalk.sql.integrations.chalk_query import ChalkQuery
-from chalk.sql.integrations.cloudsql import CloudSQLSourceImpl
-from chalk.sql.integrations.mysql import MySQLSourceImpl
-from chalk.sql.integrations.postgres import PostgreSQLSourceImpl
-from chalk.sql.integrations.redshift import RedshiftSourceImpl
-from chalk.sql.integrations.snowflake import SnowflakeSourceImpl
-from chalk.sql.integrations.sqlite import SQLiteFileSourceImpl, SQLiteInMemorySourceImpl
-from chalk.sql.integrations.string_chalk_query import StringChalkQuery
+from chalk.sql._internal.incremental import IncrementalSettings
+from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
+from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
+from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
+from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
+from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
+from chalk.sql._internal.integrations.snowflake import SnowflakeSourceImpl
+from chalk.sql._internal.integrations.sqlite import SQLiteFileSourceImpl, SQLiteInMemorySourceImpl
+from chalk.sql.finalized_query import FinalizedChalkQuery
+from chalk.sql.protocols import (
+    BaseSQLSourceProtocol,
+    ChalkQueryProtocol,
+    SQLSourceWithTableIngestProtocol,
+    StringChalkQueryProtocol,
+    TableIngestProtocol,
+)
 
 
 @overload
@@ -61,23 +65,23 @@ def SnowflakeSource(
 def PostgreSQLSource(
     *,
     host: str = ...,
-    port: str = ...,
+    port: Union[int, str] = ...,
     db: str = ...,
     user: str = ...,
     password: str = ...,
-) -> TableIngestProtocol:
+) -> SQLSourceWithTableIngestProtocol:
     ...
 
 
 @overload
-def PostgreSQLSource(*, name: str) -> TableIngestProtocol:
+def PostgreSQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
     ...
 
 
 def PostgreSQLSource(
     *,
     host: Optional[str] = None,
-    port: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
     db: Optional[str] = None,
     user: Optional[str] = None,
     password: Optional[str] = None,
@@ -90,41 +94,37 @@ def PostgreSQLSource(
 def MySQLSource(
     *,
     host: str = ...,
-    port: str = ...,
+    port: Union[int, str] = ...,
     db: str = ...,
     user: str = ...,
     password: str = ...,
-) -> BaseSQLSourceProtocol:
+) -> SQLSourceWithTableIngestProtocol:
     ...
 
 
 @overload
-def MySQLSource(*, name: str) -> BaseSQLSourceProtocol:
+def MySQLSource(*, name: str) -> SQLSourceWithTableIngestProtocol:
     ...
 
 
 def MySQLSource(
     *,
     host: Optional[str] = None,
-    port: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
     db: Optional[str] = None,
     user: Optional[str] = None,
     password: Optional[str] = None,
     name: Optional[str] = None,
-) -> BaseSQLSourceProtocol:
+) -> SQLSourceWithTableIngestProtocol:
     return MySQLSourceImpl(host, port, db, user, password, name)
 
 
-def SQLiteInMemorySource() -> TableIngestProtocol:
-    return SQLiteInMemorySourceImpl()
+def SQLiteInMemorySource(name: Optional[str] = None) -> SQLSourceWithTableIngestProtocol:
+    return SQLiteInMemorySourceImpl(name)
 
 
-def SQLiteFileSource(filename: Union[str, PathLike]) -> TableIngestProtocol:
-    return SQLiteFileSourceImpl(filename)
-
-
-def TempSQLiteFileSource() -> TableIngestProtocol:
-    return SQLiteFileSourceImpl(filename=tempfile.NamedTemporaryFile(delete=False).name)
+def SQLiteFileSource(filename: Union[str, PathLike], name: Optional[str] = None) -> SQLSourceWithTableIngestProtocol:
+    return SQLiteFileSourceImpl(filename, name)
 
 
 @overload
@@ -216,18 +216,19 @@ def CloudSQLSource(
 
 
 __all__ = [
-    "BaseSQLSource",
     "BaseSQLSourceProtocol",
     "BigQuerySource",
-    "ChalkQuery",
     "CloudSQLSource",
-    "IncrementalSettings",
     "MySQLSource",
     "PostgreSQLSource",
     "RedshiftSource",
     "SQLiteFileSource",
     "SQLiteInMemorySource",
-    "StringChalkQuery",
     "SnowflakeSource",
-    "TempSQLiteFileSource",
+    "FinalizedChalkQuery",
+    "ChalkQueryProtocol",
+    "StringChalkQueryProtocol",
+    "TableIngestProtocol",
+    "SQLSourceWithTableIngestProtocol",
+    "IncrementalSettings",
 ]
