@@ -6,6 +6,7 @@ import contextlib
 import contextvars
 import logging
 from typing import Any, Dict, List, Mapping, Optional
+from weakref import WeakKeyDictionary
 
 _LOGGING_CONTEXT_STACK: contextvars.ContextVar[List[Mapping[str, Any]]] = contextvars.ContextVar(
     "_LOGGING_CONTEXT_STACK"
@@ -27,7 +28,8 @@ def get_logging_context() -> Mapping[str, Any]:
     Retrieve the log context for the current python context.
     This initializes the thread-local variable if necessary.
     """
-    return _LOGGING_CONTEXT_STACK.get([{}])[-1]
+    stack = _LOGGING_CONTEXT_STACK.get([{}])
+    return stack[-1]
 
 
 def set_logging_context(_merge: bool = True, **extra: Any):
@@ -64,15 +66,21 @@ class LogWithContextFilter(logging.Filter):
     """Filter to append the ``extras`` onto the LogRecord."""
 
     def filter(self, record: logging.LogRecord) -> bool:
-        for k, v in get_logging_context().items():
+        items = get_logging_context().items()
+        for k, v in items:
             if not hasattr(record, k):
                 setattr(record, k, v)
         return True
 
 
+filtered_loggers = WeakKeyDictionary()
+
+
 def get_logger(name: Optional[str]):
     logger = logging.getLogger(name)
-    logger.addFilter(LogWithContextFilter())
+    if logger not in filtered_loggers:
+        logger.addFilter(LogWithContextFilter())
+        filtered_loggers[logger] = True
     return logger
 
 
