@@ -1,23 +1,29 @@
 import math
 import pathlib
+import tempfile
 from dataclasses import dataclass
 from datetime import datetime, timedelta, timezone
 from typing import List, Optional
 
 import pandas as pd
-import polars as pl
-import polars.testing
+import pyarrow as pa
+import pydantic
 import pytest
 from pydantic import BaseModel
 
+from chalk.features import DataFrame, Filter, after, feature, feature_time, features, has_many, has_one, unwrap_feature
+from chalk.utils.annotation_parsing import ResolverAnnotationParser
+
 try:
     import zoneinfo
 except ImportError:
     # Zoneinfo was introduced in python 3.9
     from backports import zoneinfo
 
-from chalk.features import DataFrame, Filter, after, feature, feature_time, features, has_many, has_one, unwrap_feature
-from chalk.utils.annotation_parsing import ResolverAnnotationParser
+try:
+    import polars as pl
+except ImportError:
+    pl = None
 
 
 @dataclass(frozen=True)
@@ -25,6 +31,10 @@ class Foo:
     bar: int
 
 
+class PydanticFoo(pydantic.BaseModel):
+    bar: Optional[int]
+
+
 @features
 class ToppingPrice:
     id: str
@@ -48,11 +58,13 @@ class Taco:
     hat: str
     topping_id: str
     ts: datetime = feature_time()
-    unzoned_ts: datetime = feature(dtype=pl.Datetime("us", None))
+    unzoned_ts: datetime = feature(dtype=pa.timestamp("us", None))
     nicknames: DataFrame[Nickname] = has_many(lambda: Nickname.id == Taco.id)
     topping: "Topping" = has_one(lambda: Topping.id == Taco.topping_id)
     foo: Foo
+    pydantic_foo: PydanticFoo
     foos: List[Foo]
+    float_list: List[float]
 
 
 @features
@@ -69,8 +81,28 @@ class Thing(BaseModel):
     wow: str
 
 
+@features
+class Silly:
+    id: str
+    name: str
+
+
+@pytest.mark.skipif(pl is None, reason="Polars is not installed")
 class TestDF:
+    def test_read_csv(self, tmp_path: pathlib.Path):
+        tmp = tempfile.NamedTemporaryFile()
+        with open(tmp.name, "w") as file:
+            file.write("2,joshua\n3,samantha")
+
+        values = DataFrame.read_csv(
+            tmp.name,
+            columns={0: Silly.id, 1: Silly.name},
+            has_header=False,
+        )
+        assert values[Silly.id].max() == "3"
+
     def test_dataframe_with_pydantic(self):
+        assert pl is not None
         df_type = DataFrame[Thing]
         assert df_type.columns == ()
         assert df_type.__pydantic_model__ == Thing
@@ -88,6 +120,8 @@ class TestDF:
         x.to_pandas()
 
     def test_indexing(self):
+        import polars.testing
+
         f = DataFrame({Taco.id: [1, 2, 3], Taco.price: [5, 6, 7]})
         polars.testing.assert_frame_equal(
             f.to_polars().collect()[-1],
@@ -111,6 +145,12 @@ class TestDF:
         assert (DataFrame.from_dict({Taco.id: [1, 1, 3]}) == DataFrame.from_dict({Taco.id: [1, 1, 3]})).all()
         assert not (DataFrame.from_dict({Taco.id: [1, 1, 2]}) == DataFrame.from_dict({Taco.id: [1, 1, 3]})).all()
 
+    def test_eq_ignore_column_order(self):
+        assert (
+            DataFrame.from_dict({Taco.id: [1, 3], Taco.hat: ["foo", "bar"]})
+            == DataFrame.from_dict({Taco.hat: ["foo", "bar"], Taco.id: [1, 3]})
+        ).all()
+
     def test_from_pandas(self):
         df = pd.DataFrame({Taco.id: ["1", "2", "3"], Taco.price: [5, 6, 7]})
         chalk_df = DataFrame(df)
@@ -208,10 +248,12 @@ class TestDF:
         assert (ans_4 == DataFrame({Taco.id: [2, 3]})).all()
 
     def test_empty_struct(self):
+        assert pl is not None
         df = DataFrame({Taco.foo: []})
         assert df.to_polars().schema == {"taco.foo": pl.Struct([pl.Field("bar", pl.Int64)])}
 
     def test_null_struct(self):
+        assert pl is not None
         df = DataFrame({Taco.foo: [None]})
         assert df.to_polars().schema == {"taco.foo": pl.Struct([pl.Field("bar", pl.Int64)])}
         # Polars does not distinguish between a null entry for a struct and a struct containing all null values
@@ -236,17 +278,47 @@ class TestDF:
         assert df.to_polars().schema == {"taco.foo": pl.Struct([pl.Field("bar", pl.Int64)])}
         assert df.to_polars().collect().rows() == [({"bar": None},), ({"bar": 1},)]
 
+    def test_null_and_valid_pydantic_struct(self):
+        # Test a null value before the first value
+        df = DataFrame(
+            {
+                Taco.pydantic_foo: [
+                    PydanticFoo(bar=None),
+                    PydanticFoo(bar=1),
+                ]
+            }
+        )
+        assert df.to_polars().schema == {"taco.pydantic_foo": pl.Struct([pl.Field("bar", pl.Int64)])}
+        assert df.to_polars().collect().rows() == [({"bar": None},), ({"bar": 1},)]
+
     @pytest.mark.xfail(reason="Lists of structs are not fully supported")
-    def test_null_list(self):
-        # None for the list element should be equivalent to an empty list
+    def test_null_list_struct(self):
+        assert pl is not None
         df = DataFrame({Taco.foos: [None]})
         assert df.to_polars().schema == {"taco.foos": pl.List(pl.Struct([pl.Field("bar", pl.Int64)]))}
-        assert df.to_polars().collect().rows() == [
-            ([],),
-        ]
+        assert df.to_polars().collect().rows() == [(None,)]
+
+    def test_empty_list_struct(self):
+        assert pl is not None
+        df = DataFrame({Taco.foos: [[]]})
+        assert df.to_polars().schema == {"taco.foos": pl.List(pl.Struct([pl.Field("bar", pl.Int64)]))}
+        assert df.to_polars().collect().rows() == [([],)]
+
+    def test_null_list_float(self):
+        assert pl is not None
+        df = DataFrame({Taco.float_list: [None]})
+        assert df.to_polars().schema == {"taco.float_list": pl.List(pl.Float64)}
+        assert df.to_polars().collect().rows() == [(None,)]
+
+    def test_empty_list_float(self):
+        assert pl is not None
+        df = DataFrame({Taco.float_list: [[]]})
+        assert df.to_polars().schema == {"taco.float_list": pl.List(pl.Float64)}
+        assert df.to_polars().collect().rows() == [([],)]
 
     @pytest.mark.xfail(reason="Lists of structs are not fully supported")
     def test_list_of_null_struct(self):
+        assert pl is not None
         # None for the struct element should be a list of one empty struct
         df = DataFrame({Taco.foos: [[None]]})
         assert df.to_polars().schema == {"taco.foos": pl.List(pl.Struct([pl.Field("bar", pl.Int64)]))}
@@ -256,7 +328,7 @@ class TestDF:
 
     @pytest.mark.xfail(reason="Lists of structs are not fully supported")
     def test_list_of_not_null_struct(self):
-        # Test a null value before the first value
+        assert pl is not None
         df = DataFrame({Taco.foos: [[Foo(bar=1)]]})
         assert df.to_polars().schema == {"taco.foos": pl.List(pl.Struct([pl.Field("bar", pl.Int64)]))}
         assert df.to_polars().collect().rows() == [
@@ -265,6 +337,7 @@ class TestDF:
 
     @pytest.mark.xfail(reason="Lists of structs are not fully supported")
     def test_list_of_null_and_not_null_struct(self):
+        assert pl is not None
         # Test a null value before the first value
         df = DataFrame({Taco.foos: [[None, Foo(bar=1)]]})
         assert df.to_polars().schema == {"taco.foos": pl.List(pl.Struct([pl.Field("bar", pl.Int64)]))}
@@ -273,6 +346,7 @@ class TestDF:
         ]
 
     def test_missing_and_struct(self):
+        assert pl is not None
         # Test a null value before the first value
         df = DataFrame(
             {
@@ -324,6 +398,7 @@ class TestDF:
         assert len(tacos) == 1
 
     def test_after(self):
+        assert pl is not None
         now = datetime.now()
         data = [now - timedelta(days=10), now]
         series = pl.Series(values=data, dtype=pl.Datetime)
@@ -406,6 +481,8 @@ class TestDF:
         assert chalk_df[Taco.price].min() == 100
 
     def test_indexing_many(self):
+        import polars.testing
+
         chalk_df = DataFrame(
             {
                 Taco.id: ["t_1", "t_2", "t_3"],
@@ -499,6 +576,7 @@ class TestDF:
         assert pandas_df.values.tolist() == [["hi"]]
 
     def test_wrong_dtype(self):
+        assert pl is not None
         with pytest.raises(
             TypeError,
             match="Feature `topping.value` at index 0 with value 'not_an_int' could not be converted to dtype `i64`",
@@ -509,14 +587,9 @@ class TestDF:
         with pytest.raises(TypeError, match="Values for feature `topping.value` could not be converted to dtype `i64`"):
             # Should also raise if passing in another dataframe
             DataFrame(pl.DataFrame({"topping.value": ["not_an_int"]}))
-
-        lf = DataFrame(pl.DataFrame({"topping.value": ["not_an_int"]}).lazy())
-        with pytest.raises(
-            pl.ComputeError,
-        ):
-            # But if passing a lazy frame, we won't get the error until we collect
-            # And it will be a generic polars ComputeError
-            lf.to_polars().collect()
+        with pytest.raises(TypeError, match="Values for feature `topping.value` could not be converted to dtype `i64`"):
+            # If there is a type mismatch when passing in a lazyframe, we should materialize it and convert it immediately
+            DataFrame(pl.DataFrame({"topping.value": ["not_an_int"]}).lazy())
 
     def test_item(self):
         df = DataFrame({Topping.value: [2]})
