@@ -5,8 +5,13 @@ import pytest
 
 import chalk
 
+try:
+    import polars as pl
+except ImportError:
+    pl = None
 
-@pytest.mark.skip
+
+@pytest.mark.skipif(pl is None, reason="Polars must be installed to generate all stubs")
 def test_stubgen():
     """Test that the chalkpy stubgen runs and that the generated stub file is the one that is committed."""
     stub_file_path = os.path.join(
